package visitor._2_multi_dispatch.example;

import java.util.ArrayList;
import java.util.List;

import visitor._2_multi_dispatch.shared.ABProcessor;
import visitor._2_multi_dispatch.shared.AVisitable;
import visitor._2_multi_dispatch.shared.BVisitable;
import visitor._2_multi_dispatch.to_visit.A1;
import visitor._2_multi_dispatch.to_visit.A2;
import visitor._2_multi_dispatch.to_visit.B1;
import visitor._2_multi_dispatch.to_visit.B2;

/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Instanzen der
 * abstrakten Klassen
 * {@link AVisitable} ({@link A1} oder {@link A2})
 * sowie
 * {@link BVisitable} ({@link B1} oder {@link B2}).
 * 
 * @author Heiner K&uuml;cker
 */
final class ExampleABProcessor
extends ABProcessor
{
    /**
     * Ergebnis-Sammler-Liste
     */
    final List<String> resultList = new ArrayList<>();

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link A1} und {@link B1}.
     * 
     * @see ABProcessor#processA1B1
     */
    @Override
    public void processA1B1(
            final A1 a1 ,
            final B1 b1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                a1.getClass().getSimpleName() +
                "-" +
                b1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link A1} und {@link B2}.
     * 
     * @see ABProcessor#processA1B2
     */
    @Override
    public void processA1B2(
            final A1 a1 ,
            final B2 b2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                a1.getClass().getSimpleName() +
                "-" +
                b2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link A2} und {@link B1}.
     * 
     * @see ABProcessor#processA2B1
     */
    @Override
    public void processA2B1(
            final A2 a2 ,
            final B1 b1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                a2.getClass().getSimpleName() +
                "-" +
                b1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link A2} und {@link B2}.
     * 
     * @see ABProcessor#processA2B2
     */
    @Override
    public void processA2B2(
            final A2 a2 ,
            final B2 b2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                a2.getClass().getSimpleName() +
                "-" +
                b2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

}
