package visitor._2_multi_dispatch.shared;

import visitor._2_multi_dispatch.to_visit.A1;
import visitor._2_multi_dispatch.to_visit.B1;
import visitor._2_multi_dispatch.to_visit.B2;

/**
 * Eine Instanz dieser Klasse oder
 * der Klasse {@link A2BVisitor}
 * wird verwendet, um den Dispatch
 * je nach konkreter(finaler)
 * Implementierungsklasse des
 * Interface {@link AVisitable} 
 * zu realisieren.
 * 
 * @author Heiner K&uuml;cker
 */
final class A1BVisitor
implements BVisitor
{
    /**
     * zu besuchendes Objekt
     */
    private final A1 a1;

    /**
     * Verarbeiter-Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    private final ABProcessor abProcessor;

    /**
     * Konstruktor.
     * 
     * @param a1 zu besuchendes {@link A1}-Objekt
     * @param abProcessor Verarbeiter-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    A1BVisitor(
            final A1 a1 ,
            final ABProcessor abProcessor )
    {
        this.a1 = a1;

        if ( abProcessor == null )
        {
            throw new IllegalArgumentException(
                    "abProcessor is null" );
        }

        this.abProcessor = abProcessor;
    }

    /**
     * @see BVisitor#visitB1
     */
    @Override
    public void visitB1(
            final B1 b1 )
    {
        this.abProcessor.processA1B1(
                this.a1 ,
                b1 );
    }

    /**
     * @see BVisitor#visitB2
     */
    @Override
    public void visitB2(
            final B2 b2 )
    {
        this.abProcessor.processA1B2(
                this.a1 ,
                b2 );
    }
}