package visitor._2_multi_dispatch.shared;

import visitor._2_multi_dispatch.to_visit.A2;
import visitor._2_multi_dispatch.to_visit.B1;
import visitor._2_multi_dispatch.to_visit.B2;

/**
 * Eine Instanz dieser Klasse oder
 * der Klasse {@link A1BVisitor}
 * wird verwendet, um den Dispatch
 * je nach konkreter(finaler)
 * Implementierungsklasse des
 * Interface {@link AVisitable} 
 * zu realisieren.
 * 
 * @author Heiner K&uuml;cker
 */
final class A2BVisitor
implements BVisitor
{
    /**
     * zu besuchendes Objekt
     */
    private final A2 a2;

    /**
     * Verarbeiter-Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    private final ABProcessor abProcessor;

    /**
     * Konstruktor.
     * 
     * @param a2 zu besuchendes {@link A2}-Objekt
     * @param abProcessor Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    A2BVisitor(
            final A2 a2 ,
            final ABProcessor abProcessor )
    {
        this.a2 = a2;

        if ( abProcessor == null )
        {
            throw new IllegalArgumentException(
                    "abProcessor is null" );
        }

        this.abProcessor = abProcessor;
    }

    /**
     * @see BVisitor#visitB1
     */
    @Override
    public void visitB1(
            final B1 b1 )
    {
        this.abProcessor.processA2B1(
                this.a2 ,
                b1 );
    }

    /**
     * @see BVisitor#visitB2
     */
    @Override
    public void visitB2(
            final B2 b2 )
    {
        this.abProcessor.processA2B2(
                this.a2 ,
                b2 );
    }
}