package visitor._2_multi_dispatch.shared;

import visitor._2_multi_dispatch.to_visit.A1;
import visitor._2_multi_dispatch.to_visit.A2;
import visitor._2_multi_dispatch.to_visit.B1;
import visitor._2_multi_dispatch.to_visit.B2;



/**
 * Vom Benutzer des Visitors zu implementierende 
 * Verabeiter-Multi-Dispatch-Visitor-Klasse zum 
 * verarbeiten der besuchten konkreten 
 * Implementierungsklassen des 
 * Interfaces {@link AVisitable} sowie
 * {@link BVisitable}.
 * 
 * @author Heiner K&uuml;cker
 */
abstract public class ABProcessor
{
    /**
     * Aufzurufende Methode zum Besuchen
     * der konkreten zu besuchenden Klassen.
     * 
     * @param a
     * @param b
     */
    public final void accept(
            final AVisitable a ,
            final BVisitable b )
    {
        /* Visitor-Switch-Objekt fr konkrete Implementierungsklassen des Interface {@link AVisitable}.
         * Beginn der Visitor-Hierarchie/Kette.
         */
        final ASwitchVisitor aSwitchVisitor =
                new ASwitchVisitor(
                        //abProcessor
                        this );

        // der nachfolgende Visitor wird in diesem Aufruf im ASwitchVisitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        a.accept(
                aSwitchVisitor );

        final BVisitor bVisitor =
                aSwitchVisitor.bVisitor;

        b.accept(
                bVisitor );
    }

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link A1}
     * und
     * {@link B1}.
     * 
     * @param a1 Instanz der Klasse {@link A1}
     * @param b1 Instanz der Klasse {@link B1}
     */
    abstract public void processA1B1(
            final A1 a1 ,
            final B1 b1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link A1}
     * und
     * {@link B2}.
     * 
     * @param a1 Instanz der Klasse {@link A1}
     * @param b2 Instanz der Klasse {@link B2}
     */
    abstract public void processA1B2(
            final A1 a1 ,
            final B2 b2 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link A2}
     * und
     * {@link B1}.
     * 
     * @param a2 Instanz der Klasse {@link A2}
     * @param b1 Instanz der Klasse {@link B1}
     */
    abstract public void processA2B1(
            final A2 a2 ,
            final B1 b1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link A2}
     * und
     * {@link B2}.
     * 
     * @param a2 Instanz der Klasse {@link A2}
     * @param b2 Instanz der Klasse {@link B2}
     */
    abstract public void processA2B2(
            final A2 a2 ,
            final B2 b2 );

}
