package visitor._2_multi_dispatch.shared;

import visitor._2_multi_dispatch.to_visit.A1;
import visitor._2_multi_dispatch.to_visit.A2;
import visitor._2_multi_dispatch.to_visit.B1;
import visitor._2_multi_dispatch.to_visit.B2;


/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Instanzen des
 * Interface {@link AVisitable}
 * zur Realisierung des Multi-Dispatch.
 * 
 * @author Heiner K&uuml;cker
 */
final class ASwitchVisitor
implements AVisitor
{
    /**
     * Vom Benutzer zu implementierende Multi-Dispatch-Visitor-Klasse
     * fr Ableitungen der Interfaces {@link AVisitable} sowie
     * {@link BVisitable}.
     */
    private final ABProcessor abMultiVisitor;
    
    /**
     * Vernderliche Member zum Vermerken des
     * ausgwhlten nchsten Visitors.
     * <br/>
     * Dies erlaubt im Interface {@link AVisitor}
     * wie in vielen Visitor-Beispielen
     * gezeigt, die Verwendung von void-Methoden.
     * <br/>
     * Falls doch noch ein Return-Wert in die
     * visit-Methoden eingefhrt wird, soll
     * dieser fr den Benutzer des Visitors
     * reserviert bleiben.
     */
    BVisitor bVisitor;

    /**
     * Konstruktor.
     * 
     * @param abMultiVisitor
     */
    public ASwitchVisitor(
            final ABProcessor abMultiVisitor )
    {
        if ( abMultiVisitor == null )
        {
            throw new IllegalArgumentException(
                    "abMultiVisitor is null" );
        }
        this.abMultiVisitor = abMultiVisitor;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link A1}.
     * 
     * Hier wird entschieden, dass das {@link BVisitable}-Objekt ({@link B1} oder {@link B2})
     * mit einem {@link A1}-Objekt verknpft wird.
     * 
     * @see AVisitor#visitA1
     */
    @Override
    public final void visitA1(
            final A1 a1 )
    {
        final A1BVisitor a1bVisitor =
                new A1BVisitor(
                        a1 ,
                        this.abMultiVisitor );

        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.bVisitor = a1bVisitor;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link A2}.
     * 
     * Hier wird entschieden, dass das {@link BVisitable}-Objekt ({@link B1} oder {@link B2})
     * mit einem {@link A2}-Objekt verknpft wird.
     * 
     * @see AVisitor#visitA2
     */
    @Override
    public final void visitA2(
            final A2 a2 )
    {
        final A2BVisitor a2bVisitor =
                new A2BVisitor(
                        a2 ,
                        this.abMultiVisitor );

        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.bVisitor = a2bVisitor;
    }

}
