package visitor._3_multi_dispatch.example;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import visitor._3_multi_dispatch.shared.CVisitable;
import visitor._3_multi_dispatch.shared.DVisitable;
import visitor._3_multi_dispatch.shared.EVisitable;
import visitor._3_multi_dispatch.to_visit.C1;
import visitor._3_multi_dispatch.to_visit.C2;
import visitor._3_multi_dispatch.to_visit.D1;
import visitor._3_multi_dispatch.to_visit.D2;
import visitor._3_multi_dispatch.to_visit.E1;
import visitor._3_multi_dispatch.to_visit.E2;

/**
 * JUnit4-Testcase fr die Klasse {@link ExampleCDEProcessor}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ExampleCDEMultiVisitorTest
{
    /**
     * Test method for {@link ExampleCDEProcessor}.
     */
    @Test
    @SuppressWarnings( "static-method" )
    public final void test()
    {
        // zur Vereinfachung wird statt eines Baumes nur eine Liste der besuchbaren Objekte aufgebaut
        final List<? extends CVisitable> cListToVisit =
                Arrays.asList(
                        new C1() ,
                        new C2() );

        // zur Vereinfachung wird statt eines Baumes nur eine Liste der besuchbaren Objekte aufgebaut
        final List<? extends DVisitable> dListToVisit =
                Arrays.asList(
                        new D1() ,
                        new D2() );

        // zur Vereinfachung wird statt eines Baumes nur eine Liste der besuchbaren Objekte aufgebaut
        final List<? extends EVisitable> eListToVisit =
                Arrays.asList(
                        new E1() ,
                        new E2() );

        final ExampleCDEProcessor cdeProcessor =
                new ExampleCDEProcessor();

        for ( final CVisitable c : cListToVisit )
        {
            for ( final DVisitable d : dListToVisit )
            {
                for ( final EVisitable e : eListToVisit )
                {
                    cdeProcessor.accept(
                            c ,
                            d ,
                            e );
                }
            }
        }

        Assert.assertEquals(
                //expected
                new ArrayList<>(
                        Arrays.asList(
                                "processC1D1E1:" + ( new C1() ).getClass().getSimpleName() + "-" + ( new D1() ).getClass().getSimpleName() + "-" + ( new E1() ).getClass().getSimpleName() ,
                                "processC1D1E2:" + ( new C1() ).getClass().getSimpleName() + "-" + ( new D1() ).getClass().getSimpleName() + "-" + ( new E2() ).getClass().getSimpleName() ,
                                "processC1D2E1:" + ( new C1() ).getClass().getSimpleName() + "-" + ( new D2() ).getClass().getSimpleName() + "-" + ( new E1() ).getClass().getSimpleName() ,
                                "processC1D2E2:" + ( new C1() ).getClass().getSimpleName() + "-" + ( new D2() ).getClass().getSimpleName() + "-" + ( new E2() ).getClass().getSimpleName() ,
                                "processC2D1E1:" + ( new C2() ).getClass().getSimpleName() + "-" + ( new D1() ).getClass().getSimpleName() + "-" + ( new E1() ).getClass().getSimpleName() ,
                                "processC2D1E2:" + ( new C2() ).getClass().getSimpleName() + "-" + ( new D1() ).getClass().getSimpleName() + "-" + ( new E2() ).getClass().getSimpleName() ,
                                "processC2D2E1:" + ( new C2() ).getClass().getSimpleName() + "-" + ( new D2() ).getClass().getSimpleName() + "-" + ( new E1() ).getClass().getSimpleName() ,
                                "processC2D2E2:" + ( new C2() ).getClass().getSimpleName() + "-" + ( new D2() ).getClass().getSimpleName() + "-" + ( new E2() ).getClass().getSimpleName() ) ) ,
                //actual
                cdeProcessor.resultList );
    }
}
