package visitor._3_multi_dispatch.example;

import java.util.ArrayList;
import java.util.List;

import visitor._3_multi_dispatch.shared.CDEProcessor;
import visitor._3_multi_dispatch.shared.CVisitable;
import visitor._3_multi_dispatch.shared.DVisitable;
import visitor._3_multi_dispatch.shared.EVisitable;
import visitor._3_multi_dispatch.to_visit.C1;
import visitor._3_multi_dispatch.to_visit.C2;
import visitor._3_multi_dispatch.to_visit.D1;
import visitor._3_multi_dispatch.to_visit.D2;
import visitor._3_multi_dispatch.to_visit.E1;
import visitor._3_multi_dispatch.to_visit.E2;

/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Implementierungs-Objekte 
 * der Interfaces
 * {@link CVisitable} ({@link C1} oder {@link C2}),
 * {@link DVisitable} ({@link D1} oder {@link D2})
 * sowie
 * {@link EVisitable} ({@link E1} oder {@link E2}).
 * 
 * @author Heiner K&uuml;cker
 */
final class ExampleCDEProcessor
extends CDEProcessor
{
    /**
     * Ergebnis-Sammler-Liste
     */
    final List<String> resultList = new ArrayList<>();

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D1}, {@link D1} und {@link E1}.
     * 
     * @see CDEProcessor#processC1D1E1
     */
    @Override
    public void processC1D1E1(
            final C1 c1 ,
            final D1 d1 ,
            final E1 e1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c1.getClass().getSimpleName() +
                "-" +
                d1.getClass().getSimpleName() +
                "-" +
                e1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D1}, {@link D1} und {@link E2}.
     * 
     * @see CDEProcessor#processC1D1E2
     */
    @Override
    public void processC1D1E2(
            final C1 c1 ,
            final D1 d1 ,
            final E2 e2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c1.getClass().getSimpleName() +
                "-" +
                d1.getClass().getSimpleName() +
                "-" +
                e2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D1}, {@link D2} und {@link E1}.
     * 
     * @see CDEProcessor#processC1D2E1
     */
    @Override
    public void processC1D2E1(
            final C1 c1 ,
            final D2 d2 ,
            final E1 e1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c1.getClass().getSimpleName() +
                "-" +
                d2.getClass().getSimpleName() +
                "-" +
                e1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D1}, {@link D2} und {@link E2}.
     * 
     * @see CDEProcessor#processC1D2E2
     */
    @Override
    public void processC1D2E2(
            final C1 c1 ,
            final D2 d2 ,
            final E2 e2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c1.getClass().getSimpleName() +
                "-" +
                d2.getClass().getSimpleName() +
                "-" +
                e2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D2}, {@link D1} und {@link E1}.
     * 
     * @see CDEProcessor#processC2D1E1
     */
    @Override
    public void processC2D1E1(
            final C2 c2 ,
            final D1 d1 ,
            final E1 e1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c2.getClass().getSimpleName() +
                "-" +
                d1.getClass().getSimpleName() +
                "-" +
                e1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D2}, {@link D1} und {@link E2}.
     * 
     * @see CDEProcessor#processC2D1E2
     */
    @Override
    public void processC2D1E2(
            final C2 c2 ,
            final D1 d1 ,
            final E2 e2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c2.getClass().getSimpleName() +
                "-" +
                d1.getClass().getSimpleName() +
                "-" +
                e2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D2}, {@link D2} und {@link E1}.
     * 
     * @see CDEProcessor#processC2D2E1
     */
    @Override
    public void processC2D2E1(
            final C2 c2 ,
            final D2 d2 ,
            final E1 e1 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c2.getClass().getSimpleName() +
                "-" +
                d2.getClass().getSimpleName() +
                "-" +
                e1.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klassen
     * {@link D2}, {@link D2} und {@link E2}.
     * 
     * @see CDEProcessor#processC2D2E2
     */
    @Override
    public void processC2D2E2(
            final C2 c2 ,
            final D2 d2 ,
            final E2 e2 )
    {
        final String resultStr =
                new Exception().getStackTrace()[ 0 ].getMethodName() +
                ":" +
                c2.getClass().getSimpleName() +
                "-" +
                d2.getClass().getSimpleName() +
                "-" +
                e2.getClass().getSimpleName();

        System.out.println( resultStr );

        this.resultList.add( resultStr );
    }

}
