package visitor._3_multi_dispatch.shared;

import visitor._3_multi_dispatch.to_visit.C1;
import visitor._3_multi_dispatch.to_visit.D2;
import visitor._3_multi_dispatch.to_visit.E1;
import visitor._3_multi_dispatch.to_visit.E2;

/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Implementierungsklassen
 * {@link E1} oder {@link E2}
 * des Interface {@link EVisitable}
 * zur Realisierung des Multi-Dispatch.
 * <br/>
 * Das vorher aufgetauchte
 * {@link C1}-Objekt und
 * {@link D2}-Objekt wurde vermerkt
 * und es wurden die passenden Methoden
 * {@link CDEProcessor#processC1D2E1}
 * {@link CDEProcessor#processC1D2E2}
 * fr den Aufruf ausgewhlt.
 * <br/>
 * In dieser Klasse wird wie im normalen
 * (double dispatch) Visitor, die
 * Methode {@link #visitE1} oder
 * Methode {@link #visitE2} mit dem
 * nchstfolgende {@link EVisitable}-Objekt
 * aufgerufen und dieses dabei mit
 * den vorher aufgetauchten
 * {@link C1}-Objekt und
 * {@link D2}-Objekt verknpft.
 * 
 * @author Heiner K&uuml;cker
 */
final class C1D2EVisitor
implements EVisitor
{
    /**
     * zu besuchendes Objekt
     */
    private final C1 c1;

    /**
     * zu besuchendes Objekt
     */
    private final D2 d2;

    /**
     * Verarbeiter-Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    private final CDEProcessor cdeProcessor;

    /**
     * Konstruktor.
     * 
     * @param c1 zu besuchendes {@link C1}-Objekt
     * @param d2 zu besuchendes {@link D2}-Objekt
     * @param cdeProcessor Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    C1D2EVisitor(
            final C1 c1 ,
            final D2 d2 ,
            final CDEProcessor cdeProcessor )
    {
        this.c1 = c1;
        this.d2 = d2;

        if ( cdeProcessor == null )
        {
            throw new IllegalArgumentException(
                    "cdeProcessor is null" );
        }

        this.cdeProcessor = cdeProcessor;
    }

    /**
     * Aufruf der Methode
     * {@link CDEProcessor#processC1D2E1}
     * mit dem {@link E1}-Objekt
     * und den vorher aufgetauchten
     * {@link C1}-Objekt und
     * {@link D2}-Objekt.
     * 
     * @see EVisitor#visitE1
     */
    @Override
    public void visitE1(
            final E1 e1 )
    {
        this.cdeProcessor.processC1D2E1(
                this.c1 ,
                this.d2 ,
                e1 );
    }

    /**
     * Aufruf der Methode
     * {@link CDEProcessor#processC1D2E2}
     * mit dem {@link E2}-Objekt
     * und den vorher aufgetauchten
     * {@link C1}-Objekt und
     * {@link D2}-Objekt.
     * 
     * @see EVisitor#visitE2
     */
    @Override
    public void visitE2(
            final E2 e2 )
    {
        this.cdeProcessor.processC1D2E2(
                this.c1 ,
                this.d2 ,
                e2 );
    }
}