package visitor._3_multi_dispatch.shared;

import visitor._3_multi_dispatch.to_visit.C2;
import visitor._3_multi_dispatch.to_visit.D1;
import visitor._3_multi_dispatch.to_visit.E1;
import visitor._3_multi_dispatch.to_visit.E2;

/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Implementierungsklassen
 * {@link E1} oder {@link E2}
 * des Interface {@link EVisitable}
 * zur Realisierung des Multi-Dispatch.
 * <br/>
 * Das vorher aufgetauchte
 * {@link C2}-Objekt und
 * {@link D1}-Objekt wurde vermerkt
 * und es wurden die passenden Methoden
 * {@link CDEProcessor#processC2D1E1}
 * {@link CDEProcessor#processC2D1E2}
 * fr den Aufruf ausgewhlt.
 * <br/>
 * In dieser Klasse wird wie im normalen
 * (double dispatch) Visitor, die
 * Methode {@link #visitE1} oder
 * Methode {@link #visitE2} mit dem
 * nchstfolgende {@link EVisitable}-Objekt
 * aufgerufen und dieses dabei mit
 * den vorher aufgetauchten
 * {@link C2}-Objekt und
 * {@link D1}-Objekt verknpft.
 * 
 * @author Heiner K&uuml;cker
 */
final class C2D1EVisitor
implements EVisitor
{
    /**
     * zu besuchendes Objekt
     */
    private final C2 c2;

    /**
     * zu besuchendes Objekt
     */
    private final D1 d1;

    /**
     * Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    private final CDEProcessor cdeProcessor;

    /**
     * Konstruktor.
     * 
     * @param c2 zu besuchendes {@link C2}-Objekt
     * @param d1 zu besuchendes {@link D1}-Objekt
     * @param cdeProcessor Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    C2D1EVisitor(
            final C2 c2 ,
            final D1 d1 ,
            final CDEProcessor cdeProcessor )
    {
        this.c2 = c2;
        this.d1 = d1;

        if ( cdeProcessor == null )
        {
            throw new IllegalArgumentException(
                    "cdeProcessor is null" );
        }

        this.cdeProcessor = cdeProcessor;
    }

    /**
     * Aufruf der Methode
     * {@link CDEProcessor#processC2D1E1}
     * mit dem {@link E1}-Objekt
     * und den vorher aufgetauchten
     * {@link C2}-Objekt und
     * {@link D1}-Objekt.
     * 
     * @see EVisitor#visitE1
     */
    @Override
    public void visitE1(
            final E1 e1 )
    {
        this.cdeProcessor.processC2D1E1(
                this.c2 ,
                this.d1 ,
                e1 );
    }

    /**
     * Aufruf der Methode
     * {@link CDEProcessor#processC2D1E2}
     * mit dem {@link E2}-Objekt
     * und den vorher aufgetauchten
     * {@link C2}-Objekt und
     * {@link D1}-Objekt.
     * 
     * @see EVisitor#visitE2
     */
    @Override
    public void visitE2(
            final E2 e2 )
    {
        this.cdeProcessor.processC2D1E2(
                this.c2 ,
                this.d1 ,
                e2 );
    }
}