package visitor._3_multi_dispatch.shared;

import visitor._3_multi_dispatch.to_visit.C2;
import visitor._3_multi_dispatch.to_visit.D1;
import visitor._3_multi_dispatch.to_visit.D2;


/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Implementierungsklassen
 * {@link D1} oder {@link D2}
 * des Interface {@link DVisitable}
 * zur Realisierung des Multi-Dispatch.
 * <br/>
 * Das vorher aufgetauchte
 * {@link C2}-Objekt wurde vermerkt
 * und es wurden die passenden Methoden
 * {@link CDEProcessor#processC2D1E1}
 * {@link CDEProcessor#processC2D1E2}
 * {@link CDEProcessor#processC2D2E1}
 * {@link CDEProcessor#processC2D2E2}
 * fr den Aufruf ausgewhlt.
 * <br/>
 * In dieser Klasse wird entschieden,
 * ob das nchstfolgende 
 * {@link EVisitable}-Objekt
 * mit einem {@link D1}-Objekt oder 
 * einem {@link D2}-Objekt verknpft
 * wird.
 * 
 * @author Heiner K&uuml;cker
 */
final class C2DSwitchVisitor
extends AbstractCDSwitchVisitor
{
    /**
     * zu besuchendes Objekt
     */
    private final C2 c2;

    /**
     * Konstruktor.
     * 
     * @param c2 zu besuchendes {@link C2}-Objekt
     * @param cdeProcessor Visitor-Instanz mit konkretem(implementiertem) Besuchs-Code
     */
    public C2DSwitchVisitor(
            final C2 c2 ,
            final CDEProcessor cdeProcessor )
    {
        super( cdeProcessor );
        this.c2 = c2;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link D1}.
     * 
     * Hier wird entschieden, dass das nchstfolgende {@link EVisitable}-Objekt
     * mit einem {@link D1}-Objekt verknpft wird.
     * 
     * @see DVisitor#visitD1
     */
    @Override
    public final void visitD1(
            final D1 d1 )
    {
        final C2D1EVisitor c2d1eVisitor =
                new C2D1EVisitor(
                        this.c2 ,
                        d1 ,
                        this.cdeProcessor );


        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.eVisitor = c2d1eVisitor;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link C2}.
     * 
     * Hier wird entschieden, dass das nchstfolgende {@link EVisitable}-Objekt
     * mit einem {@link D2}-Objekt verknpft wird.
     * 
     * @see DVisitor#visitD2
     */
    @Override
    public final void visitD2(
            final D2 d2 )
    {
        final C2D2EVisitor c2d2eVisitor =
                new C2D2EVisitor(
                        this.c2 ,
                        d2 ,
                        this.cdeProcessor );


        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.eVisitor = c2d2eVisitor;
    }

}
