package visitor._3_multi_dispatch.shared;

import visitor._3_multi_dispatch.to_visit.C1;
import visitor._3_multi_dispatch.to_visit.C2;
import visitor._3_multi_dispatch.to_visit.D1;
import visitor._3_multi_dispatch.to_visit.D2;
import visitor._3_multi_dispatch.to_visit.E1;
import visitor._3_multi_dispatch.to_visit.E2;



/**
 * Vom Benutzer des Visitors zu implementierende 
 * Verarbeiter-Multi-Dispatch-Visitor-Klasse zum 
 * verarbeiten von besuchten konkreten 
 * Implementierungsklassen des 
 * Interfaces {@link CVisitable}, 
 * {@link DVisitable} 
 * sowie {@link EVisitable}.
 * 
 * @author Heiner K&uuml;cker
 */
abstract public class CDEProcessor
{
    /**
     * Aufzurufende Methode zum Besuchen
     * der konkreten zu besuchenden Klassen.
     * 
     * @param c 
     * @param d 
     * @param e 
     * 
     */
    public final void accept(
            final CVisitable c ,
            final DVisitable d ,
            final EVisitable e )
    {
        /**
         * Visitor-Switch-Objekt fr konkrete Implementierungsklassen des Interfaces {@link CVisitable}.
         * Beginn der Visitor-Hierarchie/Kette
         */
        final CSwitchVisitor cSwitchVisitor =
                new CSwitchVisitor(
                        //cdeProcessor
                        this );
        
        // der nachfolgende Visitor wird in diesem Aufruf im ASwitchVisitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        c.accept(
                cSwitchVisitor );

        final AbstractCDSwitchVisitor dVisitor =
                cSwitchVisitor.dVisitor;

        d.accept(
                dVisitor );

        final EVisitor eVisitor =
                dVisitor.eVisitor;

        e.accept(
                eVisitor );
    }

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C1},
     * {@link D1} und
     * {@link E1}.
     * 
     * @param c1 Instanz der Klasse {@link C1}
     * @param d1 Instanz der Klasse {@link D1}
     * @param e1 Instanz der Klasse {@link E1}
     */
    abstract public void processC1D1E1(
            final C1 c1 ,
            final D1 d1 ,
            final E1 e1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D1} und
     * {@link E2}.
     * 
     * @param c1 Instanz der Klasse {@link C1}
     * @param d1 Instanz der Klasse {@link D1}
     * @param e2 Instanz der Klasse {@link E2}
     */
    abstract public void processC1D1E2(
            final C1 c1 ,
            final D1 d1 ,
            final E2 e2 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C1},
     * {@link D2} und
     * {@link E1}.
     * 
     * @param c1 Instanz der Klasse {@link C1}
     * @param d2 Instanz der Klasse {@link D2}
     * @param e1 Instanz der Klasse {@link E1}
     */
    abstract public void processC1D2E1(
            final C1 c1 ,
            final D2 d2 ,
            final E1 e1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D2} und
     * {@link E2}.
     * 
     * @param c1 Instanz der Klasse {@link C1}
     * @param d2 Instanz der Klasse {@link D2}
     * @param e2 Instanz der Klasse {@link E2}
     */
    abstract public void processC1D2E2(
            final C1 c1 ,
            final D2 d2 ,
            final E2 e2 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D1} und
     * {@link E1}.
     * 
     * @param c2 Instanz der Klasse {@link C2}
     * @param d1 Instanz der Klasse {@link D1}
     * @param e1 Instanz der Klasse {@link E1}
     */
    abstract public void processC2D1E1(
            final C2 c2 ,
            final D1 d1 ,
            final E1 e1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D1} und
     * {@link E2}.
     * 
     * @param c2 Instanz der Klasse {@link C2}
     * @param d1 Instanz der Klasse {@link D1}
     * @param e2 Instanz der Klasse {@link E2}
     */
    abstract public void processC2D1E2(
            final C2 c2 ,
            final D1 d1 ,
            final E2 e2 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D2} und
     * {@link E1}.
     * 
     * @param c2 Instanz der Klasse {@link C2}
     * @param d2 Instanz der Klasse {@link D2}
     * @param e1 Instanz der Klasse {@link E1}
     */
    abstract public void processC2D2E1(
            final C2 c2 ,
            final D2 d2 ,
            final E1 e1 );

    /**
     * Zu implementierende Methode zum Besuchen 
     * einer Instanz der Klasse
     * {@link C2},
     * {@link D2} und
     * {@link E2}.
     * 
     * @param c2 Instanz der Klasse {@link C2}
     * @param d2 Instanz der Klasse {@link D2}
     * @param e2 Instanz der Klasse {@link E2}
     */
    abstract public void processC2D2E2(
            final C2 c2 ,
            final D2 d2 ,
            final E2 e2 );

}
