package visitor._3_multi_dispatch.shared;

import visitor._3_multi_dispatch.to_visit.C1;
import visitor._3_multi_dispatch.to_visit.C2;


/**
 * Konkrete (finale) Besucher-Klasse
 * fr konkrete Implementierungsklassen
 * {@link C1} oder {@link C2}
 * des Interface {@link CVisitable}
 * zur Realisierung des Multi-Dispatch.
 * <br/>
 * In dieser Klasse wird entschieden,
 * ob das nchstfolgende 
 * {@link DVisitable}-Objekt
 * mit einem {@link C1}-Objekt oder 
 * einem {@link C2}-Objekt verknpft
 * wird.
 * 
 * @author Heiner K&uuml;cker
 */
final class CSwitchVisitor
implements CVisitor
{
    /**
     * Vom Benutzer zu implementierende Verarbeiter-Multi-Dispatch-Visitor-Klasse
     * fr Implementierungsklassen der Interfaces {@link CVisitable}, 
     * {@link DVisitable} sowie {@link EVisitable}.
     */
    private final CDEProcessor cdeProcessor;
    
    /**
     * Vernderliche Member zum Vermerken des
     * ausgwhlten nchsten Visitors.
     * <br/>
     * Dies erlaubt im Interface {@link CVisitor}
     * wie in vielen Visitor-Beispielen
     * gezeigt, die Verwendung von void-Methoden.
     * <br/>
     * Falls doch noch ein Return-Wert in die
     * visit-Methoden eingefhrt wird, soll
     * dieser fr den Benutzer des Visitors
     * reserviert bleiben.
     */
    AbstractCDSwitchVisitor dVisitor;

    /**
     * Konstruktor.
     * 
     * @param cdeProcessor
     */
    public CSwitchVisitor(
            final CDEProcessor cdeProcessor )
    {
        if ( cdeProcessor == null )
        {
            throw new IllegalArgumentException(
                    "cdeProcessor is null" );
        }
        this.cdeProcessor = cdeProcessor;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link C1}.
     * 
     * Hier wird entschieden, dass die nchstfolgende 
     * Instanz einer {@link DVisitable}-Implementierungs-Klasse
     * mit einem {@link C1}-Objekt verknpft wird.
     * 
     * @see CVisitor#visitC1
     */
    @Override
    public final void visitC1(
            final C1 c1 )
    {
        final C1DSwitchVisitor c1dSwitchVisitor =
                new C1DSwitchVisitor(
                        c1 ,
                        this.cdeProcessor );

        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.dVisitor = c1dSwitchVisitor;
    }

    /**
     * Im Visitor-Konstrukt aufzurufende Methode fr die Klasse {@link C2}.
     * 
     * Hier wird entschieden, dass die nchstfolgende 
     * Instanz einer {@link DVisitable}-Implementierungs-Klasse
     * mit einem {@link C2}-Objekt verknpft wird.
     * 
     * @see CVisitor#visitC2
     */
    @Override
    public final void visitC2(
            final C2 c2 )
    {
        final C2DSwitchVisitor c2dSwitchVisitor =
                new C2DSwitchVisitor(
                        c2 ,
                        this.cdeProcessor );

        // der nachfolgende Visitor wird in diesem Aufruf in diesem Visitor vermerkt
        // Achtung, vernderlicher State, fieser Trick
        this.dVisitor = c2dSwitchVisitor;
    }

}
